package com.retrospective.ui.regserviceclient;

import java.awt.Desktop;
import java.net.URI;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.log4j.Logger;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.centeractive.licenseclient.ActivationKey;
import com.centeractive.licenseclient.EvaluationKey;
import com.centeractive.licenseclient.ProductInformation;
import com.centeractive.licenseclient.SimpleLicenseServerResponse;
import com.centeractive.licenseclient.Utils;
import com.centeractive.licenseclient.dialog.RegistrationDialog;
import com.centeractive.licenseclient.dialog.RegistrationHelper;
import com.retrospective.RetrospectiveVersionInformation;
import com.retrospective.core.impl.CoreUtils;

public class SWTRegistrationDialog extends Dialog implements RegistrationDialog {

    private final static Logger log = Logger.getLogger(SWTRegistrationDialog.class);

    private static final int CONTINUE_EVALUTATION_ID = IDialogConstants.CLIENT_ID + 2;

    private ClosedAction closedAction = ClosedAction.NORMAL_CLOSE;

    private String activationKeyFromServer;

    private final EvaluationKey evalKey;

    private Text registrationKeyInput;
    private boolean hasEvaluationPeriodEnded = false;

    private final ProductInformation productInfo;

    private Button radioActivateEmail;
    private Button radioActivateOnline;

    private Button onlineActivationButton;
    private Label onlineActivationTextLabel;
    private Composite onlineActivationComposite;

    private Button emailActivationButton;
    private Button emailGenerateMailButton;
    private Label emailActivationDataLabel;
    private Composite emailActivationComposite;
    private StyledText emailActivationInputArea;

    public SWTRegistrationDialog(ProductInformation productInfo, Shell parentShell, EvaluationKey evalKey) {
        super(parentShell);
        this.productInfo = productInfo;
        this.evalKey = evalKey;
    }

    @Override
    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);

        final Group defaultGroup = new Group(composite, SWT.NONE);
        final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        gridData.widthHint = 600;
        defaultGroup.setLayoutData(gridData);
        defaultGroup.setLayout(new GridLayout());

        Date today = new Date();
        if (evalKey.isExpirationDateReached(today)) {
            String evalPeriodOverText = String.format(RegistrationHelper.EVAL_PERIOD_OVER_TEXT,
                    productInfo.getProductName());
            SWTDialogFactory.createLabel(defaultGroup, evalPeriodOverText);
            hasEvaluationPeriodEnded = true;
        } else {

            String evalInfoText = String.format(RegistrationHelper.EVAL_DAYS_LEFT_TEXT, productInfo.getProductName(),
                    "" + evalKey.getRestEvaluationDays(today));
            SWTDialogFactory.createLabel(defaultGroup, evalInfoText);
        }

        String evalInfoText = String.format(RegistrationHelper.EVAL_INFO_TEXT, productInfo.getProductName());
        SWTDialogFactory.createLabel(defaultGroup, evalInfoText);
        Link retrospectiveBuyLink = new Link(defaultGroup, SWT.NONE);
        retrospectiveBuyLink.setText("<a>" + productInfo.getProductBuyUrl() + "</a>");
        retrospectiveBuyLink.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                try {
                    Desktop.getDesktop().browse(new URI(productInfo.getProductBuyUrl()));
                } catch (Exception e1) {
                    log.warn("Could not open default browser", e1);
                }
            }
        });

        new Label(defaultGroup, SWT.NONE);

        Label licenseBuyLabel = new Label(defaultGroup, SWT.WRAP);
        GridDataFactory.swtDefaults().grab(true, true).hint(SWT.DEFAULT, SWT.DEFAULT).align(SWT.FILL, SWT.FILL)
                .applyTo(licenseBuyLabel);
        String licenseBuyText = String.format(RegistrationHelper.ENTER_ACTIVATION_INFO_TEXT,
                productInfo.getProductName());
        licenseBuyLabel.setText(licenseBuyText);

        SWTDialogFactory.createLabel(defaultGroup, RegistrationHelper.ENTER_ACTIVATION_TEXT);

        registrationKeyInput = SWTDialogFactory.createText(defaultGroup, "", SWT.FILL);

        GridDataFactory.swtDefaults().grab(true, false).align(SWT.FILL, SWT.BEGINNING).applyTo(registrationKeyInput);

        new Label(defaultGroup, SWT.NONE);
        radioActivateOnline = new Button(defaultGroup, SWT.RADIO);
        radioActivateOnline.setText(RegistrationHelper.ACTIVATE_ONLINE_OPTION);
        radioActivateOnline.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                radioButtonClicked();
            }
        });

        onlineActivationComposite = new Composite(defaultGroup, SWT.NONE);
        GridDataFactory.swtDefaults().grab(true, false).align(SWT.FILL, SWT.BEGINNING)
                .applyTo(onlineActivationComposite);
        onlineActivationComposite.setLayout(new GridLayout());
        onlineActivationButton = new Button(onlineActivationComposite, SWT.NONE);
        onlineActivationButton.setText(RegistrationHelper.ACTIVATE_NOW_BUTTON);
        onlineActivationButton.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                String registrationKey = registrationKeyInput.getText();
                if (StringUtils.isEmpty(registrationKey) || !isActivationKeyFormatCorrect(registrationKey)) {
                    MessageDialog.openWarning(getParentShell(), "No valid activation key",
                            "No valid activation key entered");
                } else {
                    checkServerRegistrationWithRequest();
                }
            }
        });

        onlineActivationTextLabel = new Label(onlineActivationComposite, SWT.NONE);
        onlineActivationTextLabel.setText(RegistrationHelper.ONLINE_ACTIVATION_INFO);

        new Label(defaultGroup, SWT.NONE);
        radioActivateEmail = new Button(defaultGroup, SWT.RADIO);
        radioActivateEmail.setText(RegistrationHelper.ACTIVATE_EMAIL_OPTION);
        radioActivateEmail.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                radioButtonClicked();
            }
        });

        emailActivationComposite = new Composite(defaultGroup, SWT.NONE);
        GridDataFactory.swtDefaults().grab(true, false).align(SWT.FILL, SWT.BEGINNING)
                .applyTo(emailActivationComposite);
        emailActivationComposite.setLayout(new GridLayout());

        emailGenerateMailButton = new Button(emailActivationComposite, SWT.NONE);
        emailGenerateMailButton.setText(RegistrationHelper.GENERATE_EMAIL_BUTTON);
        emailGenerateMailButton.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                String registrationKey = registrationKeyInput.getText();
                if (StringUtils.isEmpty(registrationKey) || !isActivationKeyFormatCorrect(registrationKey)) {
                    MessageDialog.openWarning(getParentShell(), "No valid activation key",
                            "No valid activation key entered");
                } else {
                    String activationKey = registrationKeyInput.getText();
                    sendSupportEmail(productInfo, activationKey);
                }
            }
        });

        emailActivationDataLabel = new Label(emailActivationComposite, SWT.WRAP);
        GridDataFactory.swtDefaults().grab(true, true).hint(SWT.DEFAULT, SWT.DEFAULT).align(SWT.FILL, SWT.FILL)
                .applyTo(emailActivationDataLabel);
        emailActivationDataLabel.setText(RegistrationHelper.PASTE_KEY_HELPER_TEXT);

        emailActivationInputArea = new StyledText(emailActivationComposite, SWT.BORDER | SWT.V_SCROLL);
        GridDataFactory.swtDefaults().grab(true, true).hint(SWT.DEFAULT, 100).align(SWT.FILL, SWT.FILL)
                .applyTo(emailActivationInputArea);

        emailActivationButton = new Button(emailActivationComposite, SWT.NONE);
        emailActivationButton.setText(RegistrationHelper.ACTIVATE_EMAIL_BUTTON);
        emailActivationButton.addSelectionListener(new SelectionAdapter() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                String activationXml = emailActivationInputArea.getText();
                if (isServerActivationKeyValidXml(activationXml)) {
                    activationKeyFromServer = activationXml;
                    MessageDialog.openInformation(getParentShell(), "Program activated",
                            "Program is activated with a valid license.");
                    closedAction = ClosedAction.REGISTRATION_SUCCESSFUL;
                    close();
                } else {
                    MessageDialog.openWarning(getParentShell(), "No valid activation data",
                            "The activation-data you entered is not valid!");
                }
            }
        });

        SWTDialogFactory.createLabel(defaultGroup,
                String.format(RegistrationHelper.CONTACT_SUPPORT_TEXT, productInfo.getSupportEmail()));

        return composite;
    }

    public static boolean isActivationKeyFormatCorrect(String key) {
        String uuidRegexPattern = "[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}";
        return key.matches(uuidRegexPattern);
    }

    private void radioButtonClicked() {
        if (radioActivateOnline.getSelection()) {
            emailActivationButton.setEnabled(false);
            emailActivationInputArea.setEnabled(false);
            emailActivationDataLabel.setEnabled(false);
            emailActivationComposite.setEnabled(false);
            emailGenerateMailButton.setEnabled(false);
            onlineActivationButton.setEnabled(true);
            onlineActivationTextLabel.setEnabled(true);
            onlineActivationComposite.setEnabled(true);
        } else if (radioActivateEmail.getSelection()) {
            emailActivationButton.setEnabled(true);
            emailActivationInputArea.setEnabled(true);
            emailActivationDataLabel.setEnabled(true);
            emailActivationComposite.setEnabled(true);
            emailGenerateMailButton.setEnabled(true);
            onlineActivationButton.setEnabled(false);
            onlineActivationTextLabel.setEnabled(false);
            onlineActivationComposite.setEnabled(false);
        }
    }

    @Override
    protected void createButtonsForButtonBar(Composite parent) {
        Button continueEvaluationButton = createButton(parent, CONTINUE_EVALUTATION_ID, "Continue evaluation", true);
        if (hasEvaluationPeriodEnded) {
            continueEvaluationButton.setEnabled(false);
        }
    }

    @Override
    protected void buttonPressed(int buttonId) {
        super.buttonPressed(buttonId);

        if (buttonId == CONTINUE_EVALUTATION_ID) {
            closedAction = ClosedAction.CONTINUE_EVALUATION;
            close();
        }
    }

    public static void sendSupportEmail(ProductInformation productInfo, String activationKey) {
        try {

            String xml = Utils.getFlatXml(RegistrationHelper.createActivationCheckXml(productInfo, activationKey));
            String mailText = "\n\nPlease do not change the text below!\n---------------\n" + xml;
            String mail = RegistrationHelper.createMailUriString(productInfo.getSupportEmail(),
                    productInfo.getSupportEmailSubject(), mailText);

            URI uriMailTo = new URI(mail);
            Desktop.getDesktop().mail(uriMailTo);
        } catch (Exception ex) {
            String message = "Could not create support email: " + ex.getMessage();
            log.warn(message, ex);
            throw new RuntimeException(message, ex);
        }
    }

    private void checkServerRegistrationWithRequest() {
        try {
            String registrationKey = registrationKeyInput.getText();

            SimpleLicenseServerResponse response = RegistrationHelper
                    .checkRegistrationKey(productInfo, registrationKey);
            if (response.statusCode == 200) {
                // everything is great
                MessageDialog.openInformation(getParentShell(), "Program activated",
                        "Program is activated with a valid license.");
                activationKeyFromServer = response.content;
                closedAction = ClosedAction.REGISTRATION_SUCCESSFUL;
                close();
            } else {
                String message = "Registration server declined registration attempt: " + response.content;
                log.warn(message);
                MessageDialog.openInformation(getParentShell(), "Registration unsuccessful", message);
            }
        } catch (Exception ex) {
            String message = "Activation check not possible: " + ex.getMessage();
            log.warn(message, ex);
            throw new RuntimeException(message, ex);
        }
    }

    public static boolean isServerActivationKeyValidXml(String text) {
        ActivationKey key = CoreUtils.unmarshalXml(ActivationKey.class, text);
        return key != null;
    }

    @Override
    public String getServerActivationKey() {
        return activationKeyFromServer;
    }

    public ClosedAction getClosedAction() {
        return closedAction;
    }

    @Override
    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText("Program activation");
    }

    @Override
    public ClosedAction showDialogAndWait(boolean limitEvaluationCheck) {
        Date today = new Date();
        if (limitEvaluationCheck
                && evalKey.getRestEvaluationDays(today) > productInfo.getLimitOfDaysToShowActivationDialog()) {
            closedAction = ClosedAction.CONTINUE_EVALUATION;
            return closedAction;
        }
        open();
        return closedAction;
    }

    public static void main(String[] args) {
        Display exampleDisplay = new Display();
        Shell exampleShell = new Shell(exampleDisplay);

        Date startDate = Utils.createDateFromISOString("2012-03-01");
        Date endDate = DateUtils.addDays(startDate, 31);
        EvaluationKey evalKey = new EvaluationKey("Retrospective", startDate, 1, "1.0.0", endDate);

        ProductInformation productInfo = RetrospectiveVersionInformation.getInstance();
        Dialog dialog = new SWTRegistrationDialog(productInfo, exampleShell, evalKey);

        dialog.open();
    }
}
